package de.lmu.ifi.medien.control;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.image.BufferedImage;

import javax.media.Buffer;
import javax.media.Effect;
import javax.media.Format;
import javax.media.ResourceUnavailableException;
import javax.media.format.RGBFormat;

import de.lmu.ifi.medien.ui.LowerThirdCodecPanel;

/**
 * The main class for the codec. Please implement the following method:
 *         <code>int process(Buffer in, Buffer out)</code> 
 * @author Sebastian Boring
 */
public class LowerThirdCodec implements Effect, LowerThirdCodecControl
{
	public static final Dimension FRAME_SIZE = new Dimension(640, 480);
	
    protected Format inputFormat;
    protected Format outputFormat;
    protected Format[] supportedInputFormats = new Format[0];
    protected Format[] supportedOutputFormats = new Format[0];
	
    protected LowerThirdCodecPanel codecPanel;
    
    protected Dimension size;
    protected BufferedImage img;
    protected int offsetX = 40;
    protected int offsetY = 410;
    
    protected double alphaChannel = 0.5;
    protected boolean active = true;
    
    public LowerThirdCodec()
    {
    	codecPanel = new LowerThirdCodecPanel(this);
    	
    	supportedInputFormats = new Format[] {
                new RGBFormat(null,
                              Format.NOT_SPECIFIED,
                              Format.byteArray,
                              Format.NOT_SPECIFIED,
                              24,
                              3, 2, 1,
                              3, Format.NOT_SPECIFIED,
                              Format.TRUE,
                              Format.NOT_SPECIFIED)
            };

    	supportedOutputFormats = new Format[] {
                new RGBFormat(null,
                              Format.NOT_SPECIFIED,
                              Format.byteArray,
                              Format.NOT_SPECIFIED,
                              24,
                              3, 2, 1,
                              3, Format.NOT_SPECIFIED,
                              Format.TRUE,
                              Format.NOT_SPECIFIED)
            };
    }
    
	public Format[] getSupportedInputFormats()
	{
		return supportedInputFormats;
	}

	public Format[] getSupportedOutputFormats(Format inputFormat)
	{
        if (inputFormat == null)
        {
            return supportedOutputFormats;
        }
        
        if (matches(inputFormat, supportedInputFormats) != null)
        {
            return new Format[] { supportedOutputFormats[0].intersects(inputFormat) };
        }
        else
        {
            return new Format[0];
        }
	}

	public void setLocation(Point location)
	{
		if(img != null)
		{
			synchronized(img)
			{
				this.offsetX = location.x;
				this.offsetY = location.y;
			}
		}
	}
	
	public Point getLocation()
	{
		return new Point(this.offsetX, this.offsetY);
	}
	
	public void setSize(Dimension size)
	{
		this.size = size;
		if(img == null)
		{
			this.img = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_ARGB);
		}
	}
	
	public Dimension getSize()
	{
		return this.size;
	}
	
	public int getWidth()
	{
		return (getSize().width);
	}
	
	public int getHeight()
	{
		return (getSize().height);
	}
	
	public void setImage(BufferedImage image)
	{
		if(img != null)
		{
			synchronized(img)
			{
				this.img = image;
			}
		}
	}
	
	public BufferedImage getImage()
	{
		if(img != null)
		{
			synchronized(img)
			{
				return img;
			}
		}
		else
		{
			return null;
		}
	}
	
	public void setActive(boolean active)
	{
		if(img != null)
		{
			synchronized(img)
			{
				this.active = active;
			}
		}
	}
	
	public boolean isActive()
	{
		return this.active;
	}
	
	public int process(Buffer in, Buffer out)
	{
		out.copy(in);
		byte[] data = (byte[]) out.getData();
	
		if(img != null && active)
		{
			synchronized(img)
			{
				// implement the overlay
			}
		}
		
		return BUFFER_PROCESSED_OK;
	}

	public Format setInputFormat(Format input)
	{
		inputFormat = input;
		return inputFormat;
	}

	public Format setOutputFormat(Format output)
	{
		if (output == null || matches(output, supportedOutputFormats) == null)
		{
            return null;
		}
		
        RGBFormat incoming = (RGBFormat) output;
        
        Dimension size = incoming.getSize();
        
        int maxDataLength = incoming.getMaxDataLength();
        int lineStride = incoming.getLineStride();
        float frameRate = incoming.getFrameRate();
        int flipped = incoming.getFlipped();

        if (size == null)
        {
            return null;
        }
        if (maxDataLength < size.width * size.height * 3)
        {
        	maxDataLength = size.width * size.height * 3;
        }
        
        if (lineStride < size.width * 3)
        {
            lineStride = size.width * 3;
        }
        
        if (flipped != Format.FALSE)
        {
            flipped = Format.FALSE;
        }
        
        outputFormat = supportedOutputFormats[0].intersects(new RGBFormat(size,
                                                        maxDataLength,
                                                        null,
                                                        frameRate,
                                                        Format.NOT_SPECIFIED,
                                                        Format.NOT_SPECIFIED,
                                                        Format.NOT_SPECIFIED,
                                                        Format.NOT_SPECIFIED,
                                                        Format.NOT_SPECIFIED,
                                                        lineStride,
                                                        Format.NOT_SPECIFIED,
                                                        Format.NOT_SPECIFIED));

        return outputFormat;
	}

	public void close() { }

	public String getName()
	{
		return "Lower Third Codec";
	}

	public void open() throws ResourceUnavailableException { }

	public void reset() { }

	public Object getControl(String className)
	{
		if(className != null && className.equals(LowerThirdCodecControl.class.getSimpleName()))
		{
			return this;
		}
		else
		{
			return null;
		}
	}

	public Object[] getControls()
	{
		return (new Object[]{this});
	}
	
	// Utility methods.
    private Format matches(Format in, Format outs[])
    {
    	for(int i=0; i<outs.length; i++)
    	{
    		if(in.matches(outs[i]))
    		{
    			return outs[i];
    		}
    	}
		return null;
    }
    
	public Component getControlComponent()
	{
		return codecPanel;
	}
}
