package de.lmu.ifi.medien.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.image.BufferedImage;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.JTextComponent;

import de.lmu.ifi.medien.control.LowerThirdCodecControl;

/**
 * The control panel for the LowerThirdCodec. Nothing to do here!
 * @author Sebastian Boring
 */
public class LowerThirdCodecPanel extends JPanel
{
	private static final long serialVersionUID = 7810250737392137356L;

	private LowerThirdCodecControl control;
	
	private JTextField line1TextField = new JTextField("Enter a person's name...", 20);
	private JTextField line2TextField = new JTextField("Enter a person's profession...", 20);
	
	private JSpinner xSpinner = new JSpinner(new SpinnerNumberModel(40, 0, 80, 1));
	private JSpinner ySpinner = new JSpinner(new SpinnerNumberModel(410, 0, 430, 1));
	private JSpinner widthSpinner = new JSpinner(new SpinnerNumberModel(560, 0, 640, 1));
	private JSpinner heightSpinner = new JSpinner(new SpinnerNumberModel(50, 0, 480, 1));
	
	private JSlider sideBarAlphaSlider = new JSlider(JSlider.VERTICAL, 0, 100, 80);
	private JLabel sideBarAlphaValueLabel = new JLabel(sideBarAlphaSlider.getValue() + " %");
	private JButton sideBarColorButton = new JButton();

	private JSlider backgroundAlphaSlider = new JSlider(JSlider.VERTICAL, 0, 100, 50);
	private JLabel backgroundAlphaValueLabel = new JLabel(backgroundAlphaSlider.getValue() + " %");
	private JButton backgroundColorButton = new JButton();
	
	private JSlider fontAlphaSlider = new JSlider(JSlider.VERTICAL, 0, 100, 100);
	private JLabel fontAlphaValueLabel = new JLabel(fontAlphaSlider.getValue() + " %");
	private JButton fontColorButton = new JButton();
	
	private JSlider masterAlphaSlider = new JSlider(JSlider.VERTICAL, 0, 100, 100);
	private JLabel masterAlphaValueLabel = new JLabel("100 %");
	private JCheckBox masterCheckBox = new JCheckBox("Active", true);
	
	private Color sideBarColor = Color.RED;
	private Color backgroundColor = Color.WHITE;
	private Color fontColor = Color.BLACK;
	
	private Font largeFont = new Font("Arial", Font.BOLD, 20);
	private Font smallFont = new Font("Arial", Font.ITALIC, 12);
	
	private BufferedImage image;
	
	public LowerThirdCodecPanel(LowerThirdCodecControl control)
	{
		this.control = control;
		
		initializePanel();
		addFocusListeners();
		addActionListeners();
		
		updateControl();
		generateOverlayImage();
	}
	
	private void updateControl()
	{
		int newWidth = ((SpinnerNumberModel)widthSpinner.getModel()).getNumber().intValue();
		int newHeight = ((SpinnerNumberModel)heightSpinner.getModel()).getNumber().intValue();
		
		control.setSize(new Dimension(newWidth, newHeight));
	}
	
	private void initializePanel()
	{
		this.setLayout(new GridBagLayout());

		//
		// Text
		//			
		initializeTextPanel();
		
		//
		// Position and Size
		//
		initializePositionAndSizePanel();

		//
		// Colors and Sliders
		//
		initializeColorAndSliderPanel();
	}
	
	private void initializeTextPanel()
	{
		JPanel textPanel = new JPanel(new GridBagLayout());
		
		JLabel textLabel = new JLabel("Text:");
		textLabel.setFont(new Font(textLabel.getFont().getFontName(), Font.BOLD, textLabel.getFont().getSize()));			
		
		JLabel line1Label = new JLabel("Line 1:");
		JLabel line2Label = new JLabel("Line 2:");
		
		textPanel.add(textLabel, makeConstraints(0, 0, 4, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, new Insets(2, 2, 6, 2)));
		
		textPanel.add(line1Label, makeConstraints(0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		textPanel.add(line1TextField, makeConstraints(1, 1, 3, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		textPanel.add(line2Label, makeConstraints(0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		textPanel.add(line2TextField, makeConstraints(1, 2, 3, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		
		this.add(textPanel, makeConstraints(0, 0, 4, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		this.add(new JSeparator(JSeparator.HORIZONTAL), makeConstraints(0, 1, 4, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST));
	}
	
	private void initializePositionAndSizePanel()
	{
		JPanel positionSizePanel = new JPanel(new GridBagLayout());
		
		JLabel positionSizeLabel = new JLabel("Position & Size:");
		positionSizeLabel.setFont(new Font(positionSizeLabel.getFont().getFontName(), Font.BOLD, positionSizeLabel.getFont().getSize()));
		
		JLabel xLabel = new JLabel("X:");
		JLabel yLabel = new JLabel("Y:");
		JLabel widthLabel = new JLabel("Width:");
		JLabel heightLabel = new JLabel("Height:");
		
		positionSizePanel.add(positionSizeLabel, makeConstraints(0, 0, 4, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, new Insets(2, 2, 6, 2)));

		xSpinner.setPreferredSize(new Dimension(50, 18));
		ySpinner.setPreferredSize(new Dimension(50, 18));
		
		positionSizePanel.add(xLabel, makeConstraints(0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		positionSizePanel.add(xSpinner, makeConstraints(1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		positionSizePanel.add(yLabel, makeConstraints(0, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		positionSizePanel.add(ySpinner, makeConstraints(1, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		
		widthSpinner.setPreferredSize(new Dimension(50, 18));
		heightSpinner.setPreferredSize(new Dimension(50, 18));
		
		positionSizePanel.add(widthLabel, makeConstraints(2, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, new Insets(2, 10, 2, 5)));
		positionSizePanel.add(widthSpinner, makeConstraints(3, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		positionSizePanel.add(heightLabel, makeConstraints(2, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, new Insets(2, 10, 2, 5)));
		positionSizePanel.add(heightSpinner, makeConstraints(3, 2, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST, new Insets(2, 2, 2, 5)));
		
		this.add(positionSizePanel, makeConstraints(0, 2, 4, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		this.add(new JSeparator(JSeparator.HORIZONTAL), makeConstraints(0, 3, 4, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST));
	}
	
	private void initializeColorAndSliderPanel()
	{
		JPanel colorSliderPanel = new JPanel(new GridBagLayout());
	
		JLabel colorSliderLabel = new JLabel("Colors & Sliders:");
		colorSliderLabel.setFont(new Font(colorSliderLabel.getFont().getFontName(), Font.BOLD, colorSliderLabel.getFont().getSize()));
		
		colorSliderPanel.add(colorSliderLabel, makeConstraints(0, 0, 3, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, new Insets(2, 2, 6, 2)));
	
		//
		// Side Bar
		//
		
		JPanel sideBarPanel = new JPanel(new GridBagLayout());
		sideBarPanel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Side"));
		
		sideBarColorButton.setBackground(sideBarColor);
		sideBarColorButton.setBorder(new LineBorder(Color.BLACK, 1));
		sideBarColorButton.setPreferredSize(new Dimension(36, 24));
		sideBarColorButton.setFocusable(false);

		sideBarAlphaSlider.setFocusable(false);

		sideBarPanel.add(sideBarAlphaValueLabel, makeConstraints(0, 0, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		sideBarPanel.add(sideBarAlphaSlider, makeConstraints(0, 1, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		sideBarPanel.add(sideBarColorButton, makeConstraints(0, 2, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));

		colorSliderPanel.add(sideBarPanel, makeConstraints(0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		
		//
		// Background Color
		//
		
		JPanel backgroundPanel = new JPanel(new GridBagLayout());
		backgroundPanel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "BG"));
		
		backgroundColorButton.setBackground(backgroundColor);
		backgroundColorButton.setBorder(new LineBorder(Color.BLACK, 1));
		backgroundColorButton.setPreferredSize(new Dimension(36, 24));
		backgroundColorButton.setFocusable(false);

		backgroundAlphaSlider.setFocusable(false);

		backgroundPanel.add(backgroundAlphaValueLabel, makeConstraints(0, 0, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		backgroundPanel.add(backgroundAlphaSlider, makeConstraints(0, 1, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		backgroundPanel.add(backgroundColorButton, makeConstraints(0, 2, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		
		colorSliderPanel.add(backgroundPanel, makeConstraints(1, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		
		//
		// Font
		//
		
		JPanel fontPanel = new JPanel(new GridBagLayout());
		fontPanel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Font"));
		
		fontColorButton.setBackground(fontColor);
		fontColorButton.setBorder(new LineBorder(Color.BLACK, 1));
		fontColorButton.setPreferredSize(new Dimension(36, 24));
		fontColorButton.setFocusable(false);

		fontAlphaSlider.setFocusable(false);

		fontPanel.add(fontAlphaValueLabel, makeConstraints(0, 0, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		fontPanel.add(fontAlphaSlider, makeConstraints(0, 1, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		fontPanel.add(fontColorButton, makeConstraints(0, 2, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		
		colorSliderPanel.add(fontPanel, makeConstraints(2, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		
		//
		// Master
		//
		
		colorSliderPanel.add(new JSeparator(JSeparator.VERTICAL), makeConstraints(3, 0, 1, 2, GridBagConstraints.VERTICAL, GridBagConstraints.CENTER));			
		
		JPanel masterPanel = new JPanel(new GridBagLayout());
		masterPanel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED), "Master"));

		masterAlphaSlider.setFocusable(false);
		masterCheckBox.setFocusable(false);

		masterPanel.add(masterAlphaValueLabel, makeConstraints(0, 0, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		masterPanel.add(masterAlphaSlider, makeConstraints(0, 1, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		masterPanel.add(masterCheckBox, makeConstraints(0, 2, 1, 1, GridBagConstraints.VERTICAL, GridBagConstraints.NORTH));
		
		colorSliderPanel.add(masterPanel, makeConstraints(4, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		
		//
		// Put all together
		//
		
		this.add(colorSliderPanel, makeConstraints(0, 4, 4, 1, GridBagConstraints.NONE, GridBagConstraints.WEST));
		this.add(new JSeparator(JSeparator.HORIZONTAL), makeConstraints(0, 5, 4, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST));
	}
	
	private GridBagConstraints makeConstraints(int x, int y, int w, int h, int fill, int anchor)
	{
		return makeConstraints(x, y, w, h, fill, anchor, new Insets(2, 2, 2, 2));
	}
	
	private GridBagConstraints makeConstraints(int x, int y, int w, int h, int fill, int anchor, Insets insets)
	{
		GridBagConstraints gbc = new GridBagConstraints();
		
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		gbc.fill = fill;
		gbc.anchor = anchor;
		gbc.insets = insets;
		gbc.weightx = 0.25;
		
		return gbc;
	}
	
	private void addFocusListeners()
	{
		FocusListener textFocusListener = new FocusListener()
		{
			public void focusGained(FocusEvent evt)
			{
				int contentLength = ((JTextComponent)evt.getSource()).getText().length();
				((JTextComponent)evt.getSource()).select(0, contentLength);
			}
			
			public void focusLost(FocusEvent evt) { }
		};
		
		line1TextField.addFocusListener(textFocusListener);
		line2TextField.addFocusListener(textFocusListener);
		
		((JSpinner.NumberEditor)xSpinner.getEditor()).getTextField().addFocusListener(textFocusListener);
		((JSpinner.NumberEditor)ySpinner.getEditor()).getTextField().addFocusListener(textFocusListener);
		((JSpinner.NumberEditor)widthSpinner.getEditor()).getTextField().addFocusListener(textFocusListener);
		((JSpinner.NumberEditor)heightSpinner.getEditor()).getTextField().addFocusListener(textFocusListener);
	}
	
	private void addActionListeners()
	{
		sideBarColorButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				Color newColor = JColorChooser.showDialog(LowerThirdCodecPanel.this, "Side Bar Color", sideBarColor);
				if(newColor != null)
				{
					sideBarColor = newColor;
					sideBarColorButton.setBackground(sideBarColor);
					
					generateOverlayImage();
				}
			}
		});
		
		backgroundColorButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				Color newColor = JColorChooser.showDialog(LowerThirdCodecPanel.this, "Background Color", backgroundColor);
				if(newColor != null)
				{
					backgroundColor = newColor;
					backgroundColorButton.setBackground(backgroundColor);
					
					generateOverlayImage();
				}
			}
		});
		
		fontColorButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				Color newColor = JColorChooser.showDialog(LowerThirdCodecPanel.this, "Font Color", fontColor);
				if(newColor != null)
				{
					fontColor = newColor;
					fontColorButton.setBackground(fontColor);
					
					generateOverlayImage();
				}
			}
		});
		
		line1TextField.addCaretListener(new CaretListener()
		{
			public void caretUpdate(CaretEvent evt)
			{
				generateOverlayImage();
			}
		});
		
		line2TextField.addCaretListener(new CaretListener()
		{
			public void caretUpdate(CaretEvent evt)
			{
				generateOverlayImage();
			}
		});
		
		ChangeListener locationChangeListener = new ChangeListener()
		{
			public void stateChanged(ChangeEvent evt)
			{
				Point location = new Point(((SpinnerNumberModel)xSpinner.getModel()).getNumber().intValue(), ((SpinnerNumberModel)ySpinner.getModel()).getNumber().intValue());
				control.setLocation(location);
				
				evaluateSpinnerValues();
			}
		};
		
		ChangeListener sizeChangeListener = new ChangeListener()
		{
			public void stateChanged(ChangeEvent evt)
			{
				evaluateSpinnerValues();
				generateOverlayImage();
			}
		};
		
		ChangeListener alphaChangeListener = new ChangeListener()
		{
			public void stateChanged(ChangeEvent evt)
			{
				JSlider slider = (JSlider)evt.getSource();
				int alphaValue = slider.getValue();
				
				if(slider != null && slider.equals(sideBarAlphaSlider))
				{
					sideBarAlphaValueLabel.setText(alphaValue + " %");
				}
				else if(slider != null && slider.equals(backgroundAlphaSlider))
				{
					backgroundAlphaValueLabel.setText(alphaValue + " %");
				}
				else if(slider != null && slider.equals(fontAlphaSlider))
				{
					fontAlphaValueLabel.setText(alphaValue + " %");
				}
				else if(slider != null && slider.equals(masterAlphaSlider))
				{
					masterAlphaValueLabel.setText(alphaValue + " %");
				}
				
				generateOverlayImage();
			}
		};
		
		masterCheckBox.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				control.setActive(masterCheckBox.isSelected());
				generateOverlayImage();
			}
		});
		
		xSpinner.addChangeListener(locationChangeListener);
		ySpinner.addChangeListener(locationChangeListener);
		widthSpinner.addChangeListener(sizeChangeListener);
		heightSpinner.addChangeListener(sizeChangeListener);
		
		sideBarAlphaSlider.addChangeListener(alphaChangeListener);
		backgroundAlphaSlider.addChangeListener(alphaChangeListener);
		fontAlphaSlider.addChangeListener(alphaChangeListener);
		masterAlphaSlider.addChangeListener(alphaChangeListener);
	}
	
	private void evaluateSpinnerValues()
	{
		//
		// X Spinner
		//
		int newXMax = 640 - control.getWidth();
		((SpinnerNumberModel)xSpinner.getModel()).setMaximum(new Integer(newXMax));

		//
		// Y Spinner
		//
		int newYMax = 480 - control.getHeight();
		((SpinnerNumberModel)ySpinner.getModel()).setMaximum(new Integer(newYMax));
		
		//
		// Width Spinner
		//
		int newWidthMax = 640 - control.getLocation().x;
		((SpinnerNumberModel)widthSpinner.getModel()).setMaximum(new Integer(newWidthMax));
		
		//
		// Width Spinner
		//
		int newHeightMax = 480 - control.getLocation().y;
		((SpinnerNumberModel)heightSpinner.getModel()).setMaximum(new Integer(newHeightMax));
	}
	
	private void generateOverlayImage()
	{
		int width = ((SpinnerNumberModel)widthSpinner.getModel()).getNumber().intValue();
		int height = ((SpinnerNumberModel)heightSpinner.getModel()).getNumber().intValue();
		
		if(image == null)
		{
			image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
			control.setImage(image);
		}
		
		if(image != null)
		{
			synchronized(image)
			{
				image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);					
				
				double masterAlphaFactor = (double)masterAlphaSlider.getValue() / 100.0;
				
				int sideBarAlpha = (int)(masterAlphaFactor * 255.0 * (double)sideBarAlphaSlider.getValue() / 100.0);
				int backgroundAlpha = (int)(masterAlphaFactor * 255.0 * (double)backgroundAlphaSlider.getValue() / 100.0);
				int fontAlpha = (int)(masterAlphaFactor * 255.0 * (double)fontAlphaSlider.getValue() / 100.0);
				
				Graphics2D g2d = image.createGraphics();
				
				g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				
				g2d.setColor(new Color(sideBarColor.getRed(), sideBarColor.getGreen(), sideBarColor.getBlue(), sideBarAlpha));
				g2d.fillRect(0, 0, 10, height);
				
				g2d.setColor(new Color(backgroundColor.getRed(), backgroundColor.getGreen(), backgroundColor.getBlue(), backgroundAlpha));
				g2d.fillRect(10, 0, width - 10, height);
				
				g2d.setColor(new Color(fontColor.getRed(), fontColor.getGreen(), fontColor.getBlue(), fontAlpha));
				g2d.setFont(largeFont);					
				g2d.drawString(line1TextField.getText(), 25, g2d.getFontMetrics(largeFont).getHeight());
				g2d.setFont(smallFont);
				g2d.drawString(line2TextField.getText(), 25, g2d.getFontMetrics(largeFont).getHeight() + g2d.getFontMetrics(smallFont).getHeight());
				
				g2d.dispose();
				
				control.setImage(image);
			}
		}
	}
}
